import 'dart:async';

typedef Callback = dynamic Function(List<dynamic>);

class EventEmitter {
  int _eventCount = 0;
  int _listenerCount = 0;
  int _maxListeners;
  final Map<String, List<int>> _active_events = {};
  final Map<String, List<int>> _removed_events = {};
  final StreamController _controller = StreamController.broadcast();

  int get eventCount => _eventCount;

  int get maxListeners => _maxListeners;

  int get listenerCount => _listenerCount;

  set maxListeners(int max) {
    _maxListeners = max;
  }

  set listenerCount(int initial) {
    _listenerCount = initial;
  }

  set eventCount(int initial) {
    _eventCount = initial;
  }

  EventEmitter({maxListeners = 50}) {
    _maxListeners = maxListeners;
  }

  void on(String eventId, Callback callback, [List<dynamic> args]) async {
    if ((_listenerCount + 1) <= _maxListeners) {
      _listenerCount++;
      _active_events.putIfAbsent(eventId, () {
        var temp = <int>[];
        temp.add(callback.hashCode);
        return temp;
      });
      await _controller.stream.listen((data) {
        if (data == eventId) {
          var _args = [];
          args == null ? _args.add(data) : _args.addAll(args);
          _args[0] != data ? _args.add(data) : null;

          if (_removed_events[eventId]?.contains(callback.hashCode) == null ||
              _removed_events[eventId].contains(callback.hashCode) == false) {
            !_active_events[eventId].contains(callback.hashCode)
                ? _active_events[eventId].add(callback.hashCode)
                : null;
            Function.apply(callback, _args);
          }
        }
      });
    } else {
      throw Exception(
          'Too many listeners assigned: Max Listeners - $maxListeners, total listeners: $_listenerCount. You can adjust by either using the Setter [maxListeners(int max)], or setting the {maxListener: } optional named parameter setting on the EventEmitter constructor.');
    }
  }

  void emit(String eventId) async {
    await _controller.add(eventId);
    _eventCount++;
  }

  void removeListener(String eventId, Function callback) async {
    await _active_events[eventId].remove(callback.hashCode);
    await _removed_events.containsKey(eventId)
        ? await _removed_events[eventId].add(callback.hashCode)
        : await (_removed_events[eventId] = []).add(callback.hashCode);
  }

  void closeAll() async {
    await _controller.close();
    _controller.isClosed
        ? null
        : throw Exception(
            'This controller ($_controller) has not been closed. Try again.');
  }
}
